@@ -3,19 +3,13 @@
 
 . $(dirname "$0")/jenkins_common.sh
 
-ENABLE_SANITIZE="--enable-sanitize"
-
-if [ "x$label" = "xFreeBSD_amd64" ]; then
-        ENABLE_SANITIZE=""
-fi
-
 src_dir="$PWD"
 build() {
     build_dir="$1"
 
     prep_build "$src_dir" "$build_dir"
 
-    "$src_dir"/configure  --disable-silent-rules --enable-static $ENABLE_SANITIZE --enable-werror \
+    "$src_dir"/configure  --disable-silent-rules --enable-static --enable-sanitize --enable-werror \
         --enable-external-tests
 
     run_make
