@@ -268,7 +268,7 @@ void osmo_tundev_free(struct osmo_tundev *tundev)
 /*! Open and configure fd of the tunnel device.
  *  \param[in] tundev The tundev object whose tunnel interface to open
  *  \param[in] flags internal linux flags to pass when creating the device (not used yet)
- *  \returns 0 on success; negative on error
+ *  \returns tun fd on success; negative on error
  */
 static int tundev_open_fd(struct osmo_tundev *tundev, int flags)
 {
@@ -489,6 +489,15 @@ const char *osmo_tundev_get_name(const struct osmo_tundev *tundev)
 	return tundev->name;
 }
 
+/*! Get FD associated to the tundev object.
+ *  \param[in] tundev The tundev object from where to retrieve the field
+ *  \returns The fd of the currently open tundev, negative otherwise
+ */
+int osmo_tundev_get_fd(const struct osmo_tundev *tundev)
+{
+	return tundev->wqueue.bfd.fd;
+}
+
 /*! Set name used to name the tunnel interface created by the tundev object.
  *  \param[in] tundev The tundev object where the field is set
  *  \param[in] dev_name The tunnel interface name to use
@@ -556,13 +565,16 @@ struct osmo_netdev *osmo_tundev_get_netdev(struct osmo_tundev *tundev)
 /*! Submit a packet to the tunnel device managed by the tundev object
  *  \param[in] tundev The tundev object owning the tunnel device where to inject the packet
  *  \param[in] msg The msgb containg the packet to transfer
- *  \returns The current value of the configured network namespace
+ *  \returns 0 on success; negative on error
  *
  * This function takes the ownership of msg in all cases.
  */
 int osmo_tundev_send(struct osmo_tundev *tundev, struct msgb *msg)
 {
-	int rc = osmo_wqueue_enqueue(&tundev->wqueue, msg);
+	int rc;
+
+	OSMO_ASSERT(tundev);
+	rc = osmo_wqueue_enqueue(&tundev->wqueue, msg);
 	if (rc < 0) {
 		LOGTUN(tundev, LOGL_ERROR, "Failed to enqueue the packet\n");
 		msgb_free(msg);
