@@ -31,6 +31,7 @@
 #include <unistd.h>
 
 #include <osmocom/ctrl/control_cmd.h>
+#include <osmocom/ctrl/control_if.h>
 
 #include <osmocom/core/msgb.h>
 #include <osmocom/core/talloc.h>
@@ -210,6 +211,16 @@ int ctrl_cmd_install(enum ctrl_node_type node, struct ctrl_cmd_element *cmd)
 {
 	vector cmds_vec;
 
+	/* If this assert triggers, it means the program forgot to initialize
+	 * the CTRL interface first by calling ctrl_handle_alloc(2)() directly
+	 * or indirectly through ctrl_interface_setup_dynip(2)()
+	 */
+	if (!ctrl_node_vec) {
+		LOGP(DLCTRL, LOGL_ERROR,
+		     "ctrl_handle must be initialized prior to installing cmds.\n");
+		return -ENODEV;
+	}
+
 	cmds_vec = vector_lookup_ensure(ctrl_node_vec, node);
 
 	if (!cmds_vec) {
@@ -414,7 +425,7 @@ struct ctrl_cmd *ctrl_cmd_parse3(void *ctx, struct msgb *msg, bool *parse_failed
 				     osmo_escape_str(var, -1));
 				goto err;
 			}
-			LOGP(DLCTRL, LOGL_DEBUG, "Command: GET %s\n", cmd->variable);
+			LOGP(DLCTRL, LOGL_DEBUG, "Command: GET %s %s\n", cmd->id, cmd->variable);
 			break;
 		case CTRL_TYPE_SET:
 			var = strtok_r(NULL, " ", &saveptr);
@@ -446,7 +457,7 @@ struct ctrl_cmd *ctrl_cmd_parse3(void *ctx, struct msgb *msg, bool *parse_failed
 				goto err;
 			}
 
-			LOGP(DLCTRL, LOGL_DEBUG, "Command: SET %s = \"%s\"\n", cmd->variable,
+			LOGP(DLCTRL, LOGL_DEBUG, "Command: SET %s %s = \"%s\"\n", cmd->id, cmd->variable,
 			     osmo_escape_str(cmd->value, -1));
 			break;
 #define REPLY_CASE(TYPE, NAME)  \
@@ -470,11 +481,11 @@ struct ctrl_cmd *ctrl_cmd_parse3(void *ctx, struct msgb *msg, bool *parse_failed
 			cmd->reply = talloc_strdup(cmd, val); \
 			if (!cmd->variable || !cmd->reply) \
 				goto oom; \
-			LOGP(DLCTRL, LOGL_DEBUG, "Command: " NAME " %s: %s\n", cmd->variable, \
+			LOGP(DLCTRL, LOGL_DEBUG, "Command: " NAME " %s %s: %s\n", cmd->id, cmd->variable, \
 			     osmo_escape_str(cmd->reply, -1)); \
 			break
-		REPLY_CASE(CTRL_TYPE_GET_REPLY, "GET REPLY");
-		REPLY_CASE(CTRL_TYPE_SET_REPLY, "SET REPLY");
+		REPLY_CASE(CTRL_TYPE_GET_REPLY, "GET_REPLY");
+		REPLY_CASE(CTRL_TYPE_SET_REPLY, "SET_REPLY");
 		REPLY_CASE(CTRL_TYPE_TRAP, "TRAP");
 #undef REPLY_CASE
 		case CTRL_TYPE_ERROR:
@@ -637,7 +648,7 @@ int ctrl_cmd_def_send(struct ctrl_cmd_def *cd)
 		cmd->type = CTRL_TYPE_ERROR;
 	}
 
-	rc = ctrl_cmd_send(&cmd->ccon->write_queue, cmd);
+	rc = ctrl_cmd_send2(cmd->ccon, cmd);
 
 	talloc_free(cmd);
 	llist_del(&cd->list);
