@@ -47,6 +47,7 @@
 #include <osmocom/core/msgb.h>
 #include <osmocom/core/socket.h>
 #include <osmocom/core/sockaddr_str.h>
+#include <osmocom/core/talloc.h>
 #include <osmocom/gsm/tlv.h>
 #include <osmocom/gprs/gprs_msgb.h>
 #include <osmocom/gprs/gprs_ns2.h>
@@ -500,7 +501,8 @@ static int add_ip4_elem(struct ns2_sns_state *gss, struct ns2_sns_elems *elems,
 {
 	/* check for duplicates */
 	for (unsigned int i = 0; i < elems->num_ip4; i++) {
-		if (memcmp(&elems->ip4[i], ip4, sizeof(*ip4)))
+		if (elems->ip4[i].ip_addr != ip4->ip_addr ||
+		    elems->ip4[i].udp_port != ip4->udp_port)
 			continue;
 		return -1;
 	}
@@ -519,8 +521,10 @@ static int remove_ip4_elem(struct ns2_sns_state *gss, struct ns2_sns_elems *elem
 	unsigned int i;
 
 	for (i = 0; i < elems->num_ip4; i++) {
-		if (memcmp(&elems->ip4[i], ip4, sizeof(*ip4)))
+		if (elems->ip4[i].ip_addr != ip4->ip_addr ||
+		    elems->ip4[i].udp_port != ip4->udp_port)
 			continue;
+
 		/* all array elements < i remain as they are; all > i are shifted left by one */
 		memmove(&elems->ip4[i], &elems->ip4[i+1], elems->num_ip4-i-1);
 		elems->num_ip4 -= 1;
@@ -573,7 +577,8 @@ static int remove_ip6_elem(struct ns2_sns_state *gss, struct ns2_sns_elems *elem
 	unsigned int i;
 
 	for (i = 0; i < elems->num_ip6; i++) {
-		if (memcmp(&elems->ip6[i], ip6, sizeof(*ip6)))
+		if (memcmp(&elems->ip6[i].ip_addr, &ip6->ip_addr, sizeof(ip6->ip_addr)) ||
+		    elems->ip6[i].udp_port != ip6->udp_port)
 			continue;
 		/* all array elements < i remain as they are; all > i are shifted left by one */
 		memmove(&elems->ip6[i], &elems->ip6[i+1], elems->num_ip6-i-1);
@@ -744,12 +749,16 @@ static int do_sns_add(struct osmo_fsm_inst *fi,
 	case AF_INET:
 		if (gss->remote.num_ip4 >= gss->num_max_ip4_remote)
 			return -NS_CAUSE_INVAL_NR_NS_VC;
+		if (!ip4)
+			return -NS_CAUSE_PROTO_ERR_UNSPEC;
 		/* TODO: log message duplicate */
 		rc = add_ip4_elem(gss, &gss->remote, ip4);
 		break;
 	case AF_INET6:
 		if (gss->remote.num_ip6 >= gss->num_max_ip6_remote)
 			return -NS_CAUSE_INVAL_NR_NS_VC;
+		if (!ip6)
+			return -NS_CAUSE_PROTO_ERR_UNSPEC;
 		/* TODO: log message duplicate */
 		rc = add_ip6_elem(gss, &gss->remote, ip6);
 		break;
@@ -787,8 +796,6 @@ static int do_sns_add(struct osmo_fsm_inst *fi,
 		break;
 	}
 
-	gprs_ns2_start_alive_all_nsvcs(nse);
-
 	return 0;
 }
 
@@ -1234,7 +1241,7 @@ static void ns2_sns_st_configured_add(struct osmo_fsm_inst *fi,
 					do_sns_delete(fi, &v4_list[j], NULL);
 				cause = -rc;
 				ns2_tx_sns_ack(gss->sns_nsvc, trans_id, &cause, NULL, 0, NULL, 0);
-				break;
+				return;
 			}
 		}
 	} else { /* IPv6 */
@@ -1255,13 +1262,15 @@ static void ns2_sns_st_configured_add(struct osmo_fsm_inst *fi,
 					do_sns_delete(fi, NULL, &v6_list[j]);
 				cause = -rc;
 				ns2_tx_sns_ack(gss->sns_nsvc, trans_id, &cause, NULL, 0, NULL, 0);
-				break;
+				return;
 			}
 		}
 	}
 
 	/* TODO: correct behaviour is to answer to the *same* NSVC from which the SNS_ADD was received */
-	ns2_tx_sns_ack(gss->sns_nsvc, trans_id, NULL, v4_list, num_v4, v6_list, num_v6);
+	ns2_tx_sns_ack(gss->sns_nsvc, trans_id, NULL, NULL, 0, NULL, 0);
+
+	gprs_ns2_start_alive_all_nsvcs(gss->nse);
 }
 
 static void ns2_sns_st_configured_delete(struct osmo_fsm_inst *fi,
@@ -1431,7 +1440,8 @@ static void ns2_sns_st_configured_change(struct osmo_fsm_inst *fi,
 		ns2_tx_sns_ack(gss->sns_nsvc, trans_id, &cause, NULL, 0, NULL, 0);
 		return;
 	}
-	ns2_tx_sns_ack(gss->sns_nsvc, trans_id, NULL, v4_list, num_v4, v6_list, num_v6);
+
+	ns2_tx_sns_ack(gss->sns_nsvc, trans_id, NULL, NULL, 0, NULL, 0);
 }
 
 static void ns2_sns_st_configured(struct osmo_fsm_inst *fi, uint32_t event, void *data)
