@@ -47,6 +47,8 @@ const struct value_string osmo_bts_features_descs[] = {
 	{ BTS_FEAT_OSMUX,		"Osmux (Osmocom RTP multiplexing)" },
 	{ BTS_FEAT_VBS,			"Voice Broadcast Service" },
 	{ BTS_FEAT_VGCS,		"Voice Group Call Service" },
+	{ BTS_FEAT_TWTS001,		"TW-TS-001 RTP format" },
+	{ BTS_FEAT_TWTS002,		"TW-TS-002 RTP format" },
 	{ 0, NULL }
 };
 
@@ -88,6 +90,8 @@ const struct value_string osmo_bts_features_names[] = {
 	{ BTS_FEAT_OSMUX, "OSMUX" },
 	{ BTS_FEAT_VBS, "VBS" },
 	{ BTS_FEAT_VGCS, "VGCS" },
+	{ BTS_FEAT_TWTS001, "TWTS001" },
+	{ BTS_FEAT_TWTS002, "TWTS002" },
 	{}
 };
 
