@@ -42,6 +42,7 @@
 #include <osmocom/gsm/protocol/gsm_08_58.h>
 #include <osmocom/gsm/protocol/gsm_04_08_gprs.h>
 #include <osmocom/gsm/protocol/gsm_23_003.h>
+#include <osmocom/gsm/protocol/gsm_44_068.h>
 
 /*! \addtogroup gsm0408
  *  @{
@@ -129,10 +130,10 @@ const struct tlv_definition gsm48_rr_att_tlvdef = {
 		[GSM48_IE_REALTIME_DIFF]	= { TLV_TYPE_TLV },
 		[GSM48_IE_START_TIME]		= { TLV_TYPE_FIXED, 2 },
 		[GSM48_IE_TIMING_ADVANCE]	= { TLV_TYPE_TV },
-		[GSM48_IE_GROUP_CIP_SEQ]	= { TLV_TYPE_SINGLE_TV },
-		[GSM48_IE_CIP_MODE_SET]		= { TLV_TYPE_SINGLE_TV },
-		[GSM48_IE_GPRS_RESUMPT]		= { TLV_TYPE_SINGLE_TV },
-		[GSM48_IE_SYNC_IND]		= { TLV_TYPE_SINGLE_TV },
+		[GSM48_IE_GROUP_CIP_SEQ_HO]	= { TLV_TYPE_SINGLE_TV },
+		[GSM48_IE_CIP_MODE_SET_HO]	= { TLV_TYPE_SINGLE_TV },
+		[GSM48_IE_GPRS_RESUMPT_HO]	= { TLV_TYPE_SINGLE_TV },
+		[GSM48_IE_SYNC_IND_HO]		= { TLV_TYPE_SINGLE_TV },
 	},
 };
 
@@ -148,7 +149,7 @@ const struct tlv_definition gsm48_mm_att_tlvdef = {
 		[GSM48_IE_NET_DST]		= { TLV_TYPE_TLV },
 
 		[GSM48_IE_LOCATION_AREA]	= { TLV_TYPE_FIXED, 5 },
-		[GSM48_IE_PRIORITY_LEV]		= { TLV_TYPE_SINGLE_TV },
+		[GSM48_IE_PRIORITY_LEV_HO]	= { TLV_TYPE_SINGLE_TV },
 		[GSM48_IE_FOLLOW_ON_PROC]	= { TLV_TYPE_T },
 		[GSM48_IE_CTS_PERMISSION]	= { TLV_TYPE_T },
 	},
@@ -221,6 +222,34 @@ char *osmo_rai_name_c(const void *ctx, const struct gprs_ra_id *rai)
 	return osmo_rai_name_buf(buf, 32, rai);
 }
 
+/*! Convert osmo_routing_area_id (new) into gprs_ra_id (old) */
+void osmo_rai_to_gprs(struct gprs_ra_id *dest, const struct osmo_routing_area_id *src)
+{
+	OSMO_ASSERT(src);
+	OSMO_ASSERT(dest);
+
+	dest->mcc = src->lac.plmn.mcc;
+	dest->mnc = src->lac.plmn.mnc;
+	dest->mnc_3_digits = src->lac.plmn.mnc_3_digits;
+
+	dest->lac = src->lac.lac;
+	dest->rac = src->rac;
+}
+
+/*! Convert gprs_ra_id (old) into osmo_routing_area_id (new) */
+void gprs_rai_to_osmo(struct osmo_routing_area_id *dest, const struct gprs_ra_id *src)
+{
+	OSMO_ASSERT(src);
+	OSMO_ASSERT(dest);
+
+	dest->lac.plmn.mcc = src->mcc;
+	dest->lac.plmn.mnc = src->mnc;
+	dest->lac.plmn.mnc_3_digits = src->mnc_3_digits;
+
+	dest->lac.lac = src->lac;
+	dest->rac = src->rac;
+}
+
 /* FIXME: convert to value_string */
 static const char *cc_state_names[32] = {
 	"NULL",
@@ -419,16 +448,54 @@ const char *gsm48_rr_msg_name(uint8_t msgtype)
 	return get_value_string(rr_msg_names, msgtype);
 }
 
+/* 3GPP TS 44.018 Table 10.4.2 */
+static const struct value_string rr_msg_type_short_names[] = {
+	{ GSM48_MT_RR_SH_SI10,		"System Information Type 10" },
+	{ GSM48_MT_RR_SH_FACCH,		"Notification/FACCH" },
+	{ GSM48_MT_RR_SH_UL_FREE,	"Uplink Free" },
+	{ GSM48_MT_RR_SH_MEAS_REP,	"Enhanced Measurement Report (uplink)" },
+	{ GSM48_MT_RR_SH_MEAS_INFO,	"Measurement Information (downlink)" },
+	{ GSM48_MT_RR_SH_VGCS_RECON,	"VBS/VGCS Reconfigure" },
+	{ GSM48_MT_RR_SH_VGCS_RECON2,	"VBS/VGCS Reconfigure2" },
+	{ GSM48_MT_RR_SH_VGCS_INFO,	"VGCS Additional Information" },
+	{ GSM48_MT_RR_SH_VGCS_SMS,	"VGCS SMS Information" },
+	{ GSM48_MT_RR_SH_SI10bis,	"System Information Type 10bis" },
+	{ GSM48_MT_RR_SH_SI10ter,	"System Information Type 10ter" },
+	{ GSM48_MT_RR_SH_VGCS_NEIGH,	"VGCS Neighbour Cell Information" },
+	{ GSM48_MT_RR_SH_APP_DATA,	"Notify Application Data" },
+	{ 0,				NULL }
+};
+
+/*! return string representation of RR Message Type using the RR short protocol discriminator */
+const char *gsm48_rr_short_pd_msg_name(uint8_t msgtype)
+{
+	return get_value_string(rr_msg_type_short_names, msgtype);
+}
 
 const struct value_string gsm48_chan_mode_names[] = {
 	{ GSM48_CMODE_SIGN,		"SIGNALLING" },
 	{ GSM48_CMODE_SPEECH_V1,	"SPEECH_V1" },
 	{ GSM48_CMODE_SPEECH_EFR,	"SPEECH_EFR" },
 	{ GSM48_CMODE_SPEECH_AMR,	"SPEECH_AMR" },
+	{ GSM48_CMODE_SPEECH_V4,	"SPEECH_V4" },
+	{ GSM48_CMODE_SPEECH_V5,	"SPEECH_V5" },
+	{ GSM48_CMODE_SPEECH_V6,	"SPEECH_V6" },
+
+	{ GSM48_CMODE_DATA_43k5_14k5,	"DATA_43k5_14k5" },
+	{ GSM48_CMODE_DATA_29k0_14k5,	"DATA_29k0_14k5" },
+	{ GSM48_CMODE_DATA_43k5_29k0,	"DATA_43k5_29k0" },
+	{ GSM48_CMODE_DATA_14k5_43k5,	"DATA_14k5_43k5" },
+	{ GSM48_CMODE_DATA_14k5_29k0,	"DATA_14k5_29k0" },
+	{ GSM48_CMODE_DATA_29k0_43k5,	"DATA_29k0_43k5" },
+
+	{ GSM48_CMODE_DATA_43k5,	"DATA_43k5" },
+	{ GSM48_CMODE_DATA_32k0,	"DATA_32k0" },
+	{ GSM48_CMODE_DATA_29k0,	"DATA_29k0" },
 	{ GSM48_CMODE_DATA_14k5,	"DATA_14k5" },
 	{ GSM48_CMODE_DATA_12k0,	"DATA_12k0" },
 	{ GSM48_CMODE_DATA_6k0,		"DATA_6k0" },
 	{ GSM48_CMODE_DATA_3k6,		"DATA_3k6" },
+
 	{ GSM48_CMODE_SPEECH_V1_VAMOS,	"SPEECH_V1_VAMOS" },
 	{ GSM48_CMODE_SPEECH_V2_VAMOS,	"SPEECH_V2_VAMOS" },
 	{ GSM48_CMODE_SPEECH_V3_VAMOS,	"SPEECH_V3_VAMOS" },
@@ -470,6 +537,8 @@ enum gsm48_chan_mode gsm48_chan_mode_to_non_vamos(enum gsm48_chan_mode mode)
 		return GSM48_CMODE_SPEECH_EFR;
 	case GSM48_CMODE_SPEECH_V3_VAMOS:
 		return GSM48_CMODE_SPEECH_AMR;
+	case GSM48_CMODE_SPEECH_V5_VAMOS:
+		return GSM48_CMODE_SPEECH_V5;
 	default:
 		return mode;
 	}
@@ -600,8 +669,10 @@ int osmo_mobile_identity_decode(struct osmo_mobile_identity *mi, const uint8_t *
 	char *str = NULL; /* initialize to avoid uninitialized false warnings on some gcc versions (11.1.0) */
 	size_t str_size = 0; /* initialize to avoid uninitialized false warnings on some gcc versions (11.1.0) */
 
-	if (!mi_data || mi_len < 1)
-		return -EBADMSG;
+	if (!mi_data || mi_len < 1) {
+		rc = -EBADMSG;
+		goto return_error;
+	}
 
 	nibbles_len = (mi_len - 1) * 2 + ((mi_data[0] & GSM_MI_ODD) ? 1 : 0);
 
@@ -823,11 +894,13 @@ int osmo_mobile_identity_encode_msgb(struct msgb *msg, const struct osmo_mobile_
  * osmo_mobile_identity.
  *
  * \param[out] mi  Return buffer for decoded Mobile Identity.
- * \param[in] msg  The Complete Layer 3 message to extract from (LU, CM Service Req or Paging Resp).
+ * \param[in] l3_data  The Complete Layer 3 message to extract from (LU, CM Service Req or Paging Resp).
+ * \param[in] l3_len  Length of l3_data in bytes.
  * \returns 0 on success, negative on error: return codes as defined in osmo_mobile_identity_decode(), or
  *          -ENOTSUP = not a Complete Layer 3 message,
  */
-int osmo_mobile_identity_decode_from_l3(struct osmo_mobile_identity *mi, struct msgb *msg, bool allow_hex)
+int osmo_mobile_identity_decode_from_l3_buf(struct osmo_mobile_identity *mi, const uint8_t *l3_data, size_t l3_len,
+					    bool allow_hex)
 {
 	const struct gsm48_hdr *gh;
 	int8_t pdisc = 0;
@@ -846,10 +919,10 @@ int osmo_mobile_identity_decode_from_l3(struct osmo_mobile_identity *mi, struct
 		.tmsi = GSM_RESERVED_TMSI,
 	};
 
-	if (msgb_l3len(msg) < sizeof(*gh))
+	if (l3_len < sizeof(*gh))
 		return -EBADMSG;
 
-	gh = msgb_l3(msg);
+	gh = (void *)l3_data;
 	pdisc = gsm48_hdr_pdisc(gh);
 	mtype = gsm48_hdr_msg_type(gh);
 
@@ -859,12 +932,12 @@ int osmo_mobile_identity_decode_from_l3(struct osmo_mobile_identity *mi, struct
 		switch (mtype) {
 		case GSM48_MT_MM_LOC_UPD_REQUEST:
 			/* First make sure that lu-> can be dereferenced */
-			if (msgb_l3len(msg) < sizeof(*gh) + sizeof(*lu))
+			if (l3_len < sizeof(*gh) + sizeof(*lu))
 				return -EBADMSG;
 
 			/* Now we know there is enough msgb data to read a lu->mi_len, so also check that */
 			lu = (struct gsm48_loc_upd_req*)gh->data;
-			if (msgb_l3len(msg) < sizeof(*gh) + sizeof(*lu) + lu->mi_len)
+			if (l3_len < sizeof(*gh) + sizeof(*lu) + lu->mi_len)
 				return -EBADMSG;
 			mi_data = lu->mi;
 			mi_len = lu->mi_len;
@@ -874,7 +947,7 @@ int osmo_mobile_identity_decode_from_l3(struct osmo_mobile_identity *mi, struct
 		case GSM48_MT_MM_CM_REEST_REQ:
 			/* Unfortunately in Phase1 the Classmark2 length is variable, so we cannot
 			 * just use gsm48_service_request struct, and need to parse it manually. */
-			if (msgb_l3len(msg) < sizeof(*gh) + 2)
+			if (l3_len < sizeof(*gh) + 2)
 				return -EBADMSG;
 
 			cm2_len = gh->data[1];
@@ -882,7 +955,7 @@ int osmo_mobile_identity_decode_from_l3(struct osmo_mobile_identity *mi, struct
 			goto got_cm2;
 
 		case GSM48_MT_MM_IMSI_DETACH_IND:
-			if (msgb_l3len(msg) < sizeof(*gh) + sizeof(*idi))
+			if (l3_len < sizeof(*gh) + sizeof(*idi))
 				return -EBADMSG;
 			idi = (struct gsm48_imsi_detach_ind*) gh->data;
 			mi_data = idi->mi;
@@ -890,7 +963,7 @@ int osmo_mobile_identity_decode_from_l3(struct osmo_mobile_identity *mi, struct
 			goto got_mi;
 
 		case GSM48_MT_MM_ID_RESP:
-			if (msgb_l3len(msg) < sizeof(*gh) + 2)
+			if (l3_len < sizeof(*gh) + 2)
 				return -EBADMSG;
 			mi_data = gh->data+1;
 			mi_len = gh->data[0];
@@ -905,13 +978,24 @@ int osmo_mobile_identity_decode_from_l3(struct osmo_mobile_identity *mi, struct
 
 		switch (mtype) {
 		case GSM48_MT_RR_PAG_RESP:
-			if (msgb_l3len(msg) < sizeof(*gh) + sizeof(*paging_response))
+			if (l3_len < sizeof(*gh) + sizeof(*paging_response))
 				return -EBADMSG;
 			paging_response = (struct gsm48_pag_resp*)gh->data;
 			cm2_len = paging_response->cm2_len;
 			cm2_buf = (uint8_t*)&paging_response->cm2;
 			goto got_cm2;
 
+		case GSM48_MT_RR_TALKER_IND:
+			/* Check minimum size: Header + CM2 LV + minimum MI LV */
+			if (l3_len < sizeof(*gh) + 4 + 2)
+				return -EBADMSG;
+			/* CM2 shall be always 3 bytes in length */
+			if (gh->data[0] != 3)
+				return -EBADMSG;
+			cm2_len = gh->data[0];
+			cm2_buf = gh->data + 1;
+			goto got_cm2;
+
 		default:
 			break;
 		}
@@ -924,7 +1008,7 @@ got_cm2:
 	/* MI (Mobile Identity) LV follows the Classmark2 */
 
 	/* There must be at least a mi_len byte after the CM2 */
-	if (cm2_buf + cm2_len + 1 > msg->tail)
+	if (cm2_buf + cm2_len + 1 > l3_data + l3_len)
 		return -EBADMSG;
 
 	mi_start = cm2_buf + cm2_len;
@@ -933,12 +1017,27 @@ got_cm2:
 
 got_mi:
 	/* mi_data points at the start of the Mobile Identity coding of mi_len bytes */
-	if (mi_data + mi_len > msg->tail)
+	if (mi_data + mi_len > l3_data + l3_len)
 		return -EBADMSG;
 
 	return osmo_mobile_identity_decode(mi, mi_data, mi_len, allow_hex);
 }
 
+/*! Extract Mobile Identity from a Complete Layer 3 message.
+ *
+ * Determine the Mobile Identity data and call osmo_mobile_identity_decode() to return a decoded struct
+ * osmo_mobile_identity.
+ *
+ * \param[out] mi  Return buffer for decoded Mobile Identity.
+ * \param[in] msg  The Complete Layer 3 message to extract from (LU, CM Service Req or Paging Resp).
+ * \returns 0 on success, negative on error: return codes as defined in osmo_mobile_identity_decode(), or
+ *          -ENOTSUP = not a Complete Layer 3 message,
+ */
+int osmo_mobile_identity_decode_from_l3(struct osmo_mobile_identity *mi, struct msgb *msg, bool allow_hex)
+{
+	return osmo_mobile_identity_decode_from_l3_buf(mi, msgb_l3(msg), msgb_l3len(msg), allow_hex);
+}
+
 /*! Return a human readable representation of a struct osmo_mobile_identity.
  * Write a string like "IMSI-1234567", "TMSI-0x1234ABCD" or "NONE", "NULL".
  * \param[out] buf  String buffer to write to.
@@ -1277,7 +1376,64 @@ int gsm48_mi_to_string(char *string, int str_len, const uint8_t *mi, int mi_len)
 	return 1;
 }
 
-/*! Parse TS 04.08 Routing Area Identifier
+/*! Decode to struct osmo_routing_area_id from a 3GPP TS 24.008 § 10.5.5.15 Routing area identification.
+ * \param[out] dst  Store the decoded result here.
+ * \param[in] ra_data  The start of a Routing Area ID in encoded form, to be decoded.
+ * \param[in] ra_data_len  Buffer size available to read from at *ra_data.
+ * \return the number of decoded bytes on success, or negative on error (if the input buffer is too small).
+ */
+int osmo_routing_area_id_decode(struct osmo_routing_area_id *dst, const uint8_t *ra_data, size_t ra_data_len)
+{
+	const struct gsm48_ra_id *ra_id;
+	if (ra_data_len < sizeof(*ra_id))
+		return -ENOSPC;
+
+	gsm48_decode_lai2((void *)ra_data, &dst->lac);
+
+	ra_id = (void *)ra_data;
+	dst->rac = ra_id->rac;
+
+	return sizeof(*ra_id);
+}
+
+/*! Encode struct osmo_routing_area_id to a 3GPP TS 24.008 § 10.5.5.15 Routing area identification: write to a buffer.
+ * \param[out] buf  Return buffer for encoded Mobile Identity.
+ * \param[in] buflen  sizeof(buf).
+ * \param[in] src  RA to encode.
+ * \return Amount of bytes written to buf, or negative on error.
+ */
+int osmo_routing_area_id_encode_buf(uint8_t *buf, size_t buflen, const struct osmo_routing_area_id *src)
+{
+	struct gsm48_ra_id *ra_id;
+	if (buflen < sizeof(*ra_id))
+		return -ENOSPC;
+
+	gsm48_generate_lai2((void *)buf, &src->lac);
+
+	ra_id = (void *)buf;
+	ra_id->rac = src->rac;
+
+	return sizeof(*ra_id);
+}
+
+/*! Encode struct osmo_routing_area_id to a 3GPP TS 24.008 § 10.5.5.15 Routing area identification: append to msgb.
+ * To succeed, the msgb must have tailroom >= sizeof(struct gsm48_ra_id).
+ * \param[out] msg  Append to this msgb.
+ * \param[in] src  Encode this Routing Area ID.
+ * \return Number of bytes appended to msgb, or negative on error.
+ */
+int osmo_routing_area_id_encode_msgb(struct msgb *msg, const struct osmo_routing_area_id *src)
+{
+	int rc = osmo_routing_area_id_encode_buf(msg->tail, msgb_tailroom(msg), src);
+	if (rc <= 0)
+		return rc;
+	msgb_put(msg, rc);
+	return rc;
+}
+
+/*! Parse TS 04.08 Routing Area Identifier.
+ * Preferably use osmo_routing_area_id_decode() instead: struct osmo_routing_area_id is better integrated with other API
+ * like osmo_plmn_cmp().
  *  \param[out] Caller-provided memory for decoded RA ID
  *  \param[in] buf Input buffer pointing to RAI IE value */
 void gsm48_parse_ra(struct gprs_ra_id *raid, const uint8_t *buf)
@@ -1361,7 +1517,7 @@ bool gsm48_ra_equal(const struct gprs_ra_id *raid1, const struct gprs_ra_id *rai
  *  Uses From Table 10.5.33 of GSM 04.08 to determine the number of
  *  paging sub-channels in the given control channel configuration
  */
-int gsm48_number_of_paging_subchannels(struct gsm48_control_channel_descr *chan_desc)
+int gsm48_number_of_paging_subchannels(const struct gsm48_control_channel_descr *chan_desc)
 {
 	unsigned int n_pag_blocks = gsm0502_get_n_pag_blocks(chan_desc);
 
@@ -1641,6 +1797,10 @@ char *gsm48_pdisc_msgtype_name_buf(char *buf, size_t buf_len, uint8_t pdisc, uin
 	case GSM48_PDISC_CC:
 		msgt_names = gsm48_cc_msgtype_names;
 		break;
+	case GSM48_PDISC_GROUP_CC:
+	case GSM48_PDISC_BCAST_CC:
+		msgt_names = osmo_gsm44068_msg_type_names;
+		break;
 	case GSM48_PDISC_NC_SS:
 		msgt_names = gsm48_nc_ss_msgtype_names;
 		break;
@@ -1694,9 +1854,13 @@ const struct value_string gsm48_reject_value_names[] = {
 	 { GSM48_REJECT_PLMN_NOT_ALLOWED, "PLMN_NOT_ALLOWED" },
 	 { GSM48_REJECT_LOC_NOT_ALLOWED, "LOC_NOT_ALLOWED" },
 	 { GSM48_REJECT_ROAMING_NOT_ALLOWED, "ROAMING_NOT_ALLOWED" },
+	 { GSM48_REJECT_NO_SUIT_CELL_IN_LA, "NO_SUIT_CELL_IN_LA" },
 	 { GSM48_REJECT_NETWORK_FAILURE, "NETWORK_FAILURE" },
+	 { GSM48_REJECT_MAC_FAILURE, "MAC_FAILURE" },
 	 { GSM48_REJECT_SYNCH_FAILURE, "SYNCH_FAILURE" },
 	 { GSM48_REJECT_CONGESTION, "CONGESTION" },
+	 { GSM48_REJECT_GSM_AUTH_UNACCEPTABLE, "GSM_AUTH_UNACCEPTABLE" },
+	 { GSM48_REJECT_NOT_AUTH_FOR_CSG, "NOT_AUTH_FOR_CSG" },
 	 { GSM48_REJECT_SRV_OPT_NOT_SUPPORTED, "SRV_OPT_NOT_SUPPORTED" },
 	 { GSM48_REJECT_RQD_SRV_OPT_NOT_SUPPORTED, "RQD_SRV_OPT_NOT_SUPPORTED" },
 	 { GSM48_REJECT_SRV_OPT_TMP_OUT_OF_ORDER, "SRV_OPT_TMP_OUT_OF_ORDER" },
@@ -1715,6 +1879,8 @@ const struct value_string gsm48_reject_value_names[] = {
 	 { GSM48_REJECT_IMPLICITLY_DETACHED, "IMPLICITLY_DETACHED" },
 	 { GSM48_REJECT_GPRS_NOT_ALLOWED_IN_PLMN, "GPRS_NOT_ALLOWED_IN_PLMN" },
 	 { GSM48_REJECT_MSC_TMP_NOT_REACHABLE, "MSC_TMP_NOT_REACHABLE" },
+	 { GSM48_REJECT_SMS_PROV_VIA_GPRS_IN_RA, "SMS_PROV_VIA_GPRS_IN_RA" },
+	 { GSM48_REJECT_NO_PDP_CONTEXT_ACTIVATED, "NO_PDP_CONTEXT_ACTIVATED" },
 	 { 0, NULL }
 };
 
