@@ -44,10 +44,11 @@ static const char bcd_num_digits[] = {
 };
 
 /*! Like gsm48_decode_bcd_number2() but with less airtight bounds checking.
- *  \param[out] Caller-provided output buffer
+ *  \param[out] output Caller-provided output buffer
+ *  \param[in] output_len sizeof(output)
  *  \param[in] bcd_lv Length-Value portion of to-be-decoded IE
  *  \param[in] h_len Length of an optional heder between L and V portion
- *  \returns - in case of success; negative on error */
+ *  \returns 0 in case of success; negative on error */
 int gsm48_decode_bcd_number(char *output, int output_len,
 			    const uint8_t *bcd_lv, int h_len)
 {
@@ -139,7 +140,7 @@ static int asc_to_bcd(const char asc)
  *  \param[in] max_len Maximum Length of \a bcd_lv
  *  \param[in] h_len Length of an optional heder between L and V portion
  *  \param[in] input phone number as 0-terminated ASCII
- *  \returns number of bytes used in \a bcd_lv
+ *  \returns number of bytes used in \a bcd_lv; negative on error
  *
  * Depending on a context (e.g. called or calling party BCD number), the
  * optional header between L and V parts can contain TON (Type Of Number),
@@ -179,8 +180,8 @@ int gsm48_encode_bcd_number(uint8_t *bcd_lv, uint8_t max_len,
 }
 
 /*! Decode TS 04.08 Bearer Capability IE (10.5.4.5)
- *  \param[out] Caller-provided memory for decoded output
- *  \[aram[in] LV portion of TS 04.08 Bearer Capability
+ *  \param[out] bcap Caller-provided memory for decoded output
+ *  \param[in] lv LV portion of TS 04.08 Bearer Capability
  *  \returns 0 on success; negative on error */
 int gsm48_decode_bearer_cap(struct gsm_mncc_bearer_cap *bcap,
 			     const uint8_t *lv)
@@ -203,7 +204,24 @@ int gsm48_decode_bearer_cap(struct gsm_mncc_bearer_cap *bcap,
 	case GSM_MNCC_BCAP_SPEECH:
 		i = 1;
 		s = 0;
-		while(!(lv[i] & 0x80)) {
+		if ((lv[1] & 0x80) != 0) { /* octet 3a is absent */
+			switch (bcap->radio) {
+			case GSM48_BCAP_RRQ_FR_ONLY:
+				bcap->speech_ver[s++] = GSM48_BCAP_SV_FR;
+				break;
+			case GSM48_BCAP_RRQ_DUAL_HR:
+				bcap->speech_ver[s++] = GSM48_BCAP_SV_HR;
+				bcap->speech_ver[s++] = GSM48_BCAP_SV_FR;
+				break;
+			case GSM48_BCAP_RRQ_DUAL_FR:
+				bcap->speech_ver[s++] = GSM48_BCAP_SV_FR;
+				bcap->speech_ver[s++] = GSM48_BCAP_SV_HR;
+				break;
+			}
+			bcap->speech_ver[s] = -1; /* end of list */
+			return 0;
+		}
+		while (!(lv[i] & 0x80)) {
 			i++; /* octet 3a etc */
 			if (in_len < i)
 				return 0;
@@ -216,9 +234,10 @@ int gsm48_decode_bearer_cap(struct gsm_mncc_bearer_cap *bcap,
 		}
 		break;
 	case GSM_MNCC_BCAP_UNR_DIG:
+	case GSM_MNCC_BCAP_AUDIO:
 	case GSM_MNCC_BCAP_FAX_G3:
 		i = 1;
-		while(!(lv[i] & 0x80)) {
+		while (!(lv[i] & 0x80)) {
 			i++; /* octet 3a etc */
 			if (in_len < i)
 				return 0;
@@ -232,7 +251,7 @@ int gsm48_decode_bearer_cap(struct gsm_mncc_bearer_cap *bcap,
 			return 0;
 		bcap->data.rate_adaption = (lv[i] >> 3) & 3;
 		bcap->data.sig_access = lv[i] & 7;
-		while(!(lv[i] & 0x80)) {
+		while (!(lv[i] & 0x80)) {
 			i++; /* octet 5a etc */
 			if (in_len < i)
 				return 0;
@@ -317,10 +336,26 @@ int gsm48_encode_bearer_cap(struct msgb *msg, int lv_only,
 		lv[i] |= 0x80; /* last IE of octet 3 etc */
 		break;
 	case GSM48_BCAP_ITCAP_UNR_DIG_INF:
+	case GSM48_BCAP_ITCAP_3k1_AUDIO:
 	case GSM48_BCAP_ITCAP_FAX_G3:
 		lv[i++] |= 0x80; /* last IE of octet 3 etc */
-		/* octet 4 */
-		lv[i++] = 0xb8;
+		/* octet 4
+		 * 1... .... = Extension: No Extension
+		 * .0.. .... = Compression: Not Allowed
+		 * ..xx .... = Structure: (see below)
+		 * .... 1... = Duplex mode: Full
+		 * .... .0.. = Configuration: Point-to-point
+		 * .... ..0. = NIRR: No meaning is associated with this value
+		 * .... ...0 = Establishment: Demand
+		 *
+		 * For connection element "non transparent":
+		 * ..00 .... = Structure: Service data unit integrity (0)
+		 * For connection element "transparent":
+		 * ..11 .... = Structure: Unstructured (3) */
+		if ((bcap->data.transp & 0x01) == 0)
+			lv[i++] = 0x88 | (0x03 << 4);
+		else
+			lv[i++] = 0x88;
 		/* octet 5 */
 		lv[i++] = 0x80 | ((bcap->data.rate_adaption & 3) << 3)
 			  | (bcap->data.sig_access & 7);
@@ -334,7 +369,9 @@ int gsm48_encode_bearer_cap(struct msgb *msg, int lv_only,
 		lv[i++] = (bcap->data.parity & 7) |
 			  ((bcap->data.interm_rate & 3) << 5);
 		/* octet 6c */
-		lv[i] = 0x80 | (bcap->data.modem_type & 0x1f);
+		lv[i] = 0x80 |
+			((bcap->data.transp & 3) << 5) |
+			(bcap->data.modem_type & 0x1f);
 		break;
 	default:
 		return -EINVAL;
@@ -350,9 +387,9 @@ int gsm48_encode_bearer_cap(struct msgb *msg, int lv_only,
 }
 
 /*! Decode TS 04.08 Call Control Capabilities IE (10.5.4.5a)
- *  \param[out] Caller-provided memory for decoded CC capabilities
+ *  \param[out] ccap Caller-provided memory for decoded CC capabilities
  *  \param[in] lv Length-Value of IE
- *  \retursns 0 on success; negative on error */
+ *  \returns 0 on success; negative on error */
 int gsm48_decode_cccap(struct gsm_mncc_cccap *ccap, const uint8_t *lv)
 {
 	uint8_t in_len = lv[0];
@@ -436,7 +473,7 @@ int gsm48_encode_called(struct msgb *msg,
 }
 
 /*! Decode TS 04.08 Caller ID
- *  \param[out] called Caller-provided memory for decoded number
+ *  \param[out] callerid Caller-provided memory for decoded number
  *  \param[in] lv Length-Value portion of IE
  *  \returns 0 on success; negative on error */
 int gsm48_decode_callerid(struct gsm_mncc_number *callerid,
@@ -808,7 +845,7 @@ int gsm48_decode_ssversion(struct gsm_mncc_ssversion *ssv,
 {
 	uint8_t in_len = lv[0];
 
-	if (in_len < 1 || in_len < sizeof(ssv->info))
+	if (in_len < 1 || in_len > sizeof(ssv->info))
 		return -EINVAL;
 
 	memcpy(ssv->info, lv + 1, in_len);
@@ -863,8 +900,9 @@ static int32_t smod(int32_t n, int32_t m)
  *  \param[in] cd Cell Channel Description IE
  *  \param[in] len Length of \a cd in bytes
  *  \returns 0 on success; negative on error */
-int gsm48_decode_freq_list(struct gsm_sysinfo_freq *f, uint8_t *cd,
-			   uint8_t len, uint8_t mask, uint8_t frqt)
+int gsm48_decode_freq_list(struct gsm_sysinfo_freq *f,
+			   const uint8_t *cd, uint8_t len,
+			   uint8_t mask, uint8_t frqt)
 {
 	int i;
 
