@@ -19,10 +19,6 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- *
  */
 
 /*! \mainpage libosmogsm Documentation
@@ -84,6 +80,7 @@
 #include <osmocom/gsm/gsm_utils.h>
 #include <osmocom/gsm/meas_rep.h>
 #include <osmocom/gsm/protocol/gsm_04_08.h>
+#include <osmocom/gsm/gsm0502.h>
 
 #include <stdlib.h>
 #include <stdint.h>
@@ -96,7 +93,7 @@
 #include <time.h>
 #include <unistd.h>
 
-#include "../../config.h"
+#include "config.h"
 
 #if (!EMBEDDED)
 /* FIXME: this can be removed once we bump glibc requirements to 2.25: */
@@ -329,12 +326,13 @@ int gsm_septet_pack(uint8_t *result, const uint8_t *rdata, size_t septet_len, ui
 	int i = 0, z = 0;
 	uint8_t cb, nb;
 	int shift = 0;
-	uint8_t *data = calloc(septet_len + 1, sizeof(uint8_t));
+	uint8_t *data = malloc(septet_len + 1);
 
 	if (padding) {
 		shift = 7 - padding;
 		/* the first zero is needed for padding */
 		memcpy(data + 1, rdata, septet_len);
+		data[0] = 0x00;
 		septet_len++;
 	} else
 		memcpy(data, rdata, septet_len);
@@ -388,7 +386,7 @@ int gsm_7bit_encode_n(uint8_t *result, size_t n, const char *data, int *octets)
 	size_t max_septets = n * 8 / 7;
 
 	/* prepare for the worst case, every character expanding to two bytes */
-	uint8_t *rdata = calloc(strlen(data) * 2, sizeof(uint8_t));
+	uint8_t *rdata = malloc(strlen(data) * 2);
 	y = gsm_septet_encode(rdata, data);
 
 	if (y > max_septets) {
@@ -888,10 +886,21 @@ char *gsm_fn_as_gsmtime_str(uint32_t fn)
 /*! Encode decoded \ref gsm_time to Frame Number
  *  \param[in] time GSM Time in decoded structure
  *  \returns GSM Frame Number */
-uint32_t gsm_gsmtime2fn(struct gsm_time *time)
+uint32_t gsm_gsmtime2fn(const struct gsm_time *time)
 {
-	/* TS 05.02 Chapter 4.3.3 TDMA frame number */
-	return (51 * ((time->t3 - time->t2 + 26) % 26) + time->t3 + (26 * 51 * time->t1));
+	uint32_t fn;
+
+	/* See also:
+	 * 3GPP TS 44.018, section 10.5.2.38, 3GPP TS 45.002 section 4.3.3, and
+	 * 3GPP TS 48.058, section 9.3.8 */
+	fn = 51 * OSMO_MOD_FLR((time->t3-time->t2), 26) + time->t3 + 51 * 26 * time->t1;
+
+	/* Note: Corrupted input values may cause a resulting frame number
+	 * larger then the maximum permitted value of GSM_MAX_FN. Even though
+	 * the caller is expected to check the input values beforehand we must
+	 * make sure that the result cannot exceed the value range of a valid
+	 * GSM frame number. */
+	return fn % GSM_MAX_FN;
 }
 
 char *osmo_dump_gsmtime_buf(char *buf, size_t buf_len, const struct gsm_time *tm)
@@ -916,6 +925,45 @@ char *osmo_dump_gsmtime_c(const void *ctx, const struct gsm_time *tm)
 	return osmo_dump_gsmtime_buf(buf, 64, tm);
 }
 
+#define GSM_RFN_THRESHOLD (GSM_RFN_MODULUS / 2)
+uint32_t gsm_rfn2fn(uint16_t rfn, uint32_t curr_fn)
+{
+	uint32_t curr_rfn;
+	uint32_t fn_rounded;
+	const uint32_t rfn32 = rfn; /* used as 32bit for calculations */
+
+	/* Ensure that all following calculations are performed with the
+	 * relative frame number */
+	OSMO_ASSERT(rfn32 < GSM_RFN_MODULUS);
+
+	/* Compute an internal relative frame number from the full internal
+	   frame number */
+	curr_rfn = gsm_fn2rfn(curr_fn);
+
+	/* Compute a "rounded" version of the internal frame number, which
+	 * exactly fits in the RFN_MODULUS raster */
+	fn_rounded = GSM_TDMA_FN_SUB(curr_fn, curr_rfn);
+
+	/* If the delta between the internal and the external relative frame
+	 * number exceeds a certain limit, we need to assume that the incoming
+	 * request belongs to a the previous rfn period. To correct this,
+	 * we roll back the rounded frame number by one RFN_MODULUS */
+	if (GSM_TDMA_FN_DIFF(rfn32, curr_rfn) > GSM_RFN_THRESHOLD) {
+		/* Race condition between rfn and curr_fn detected:  rfn belongs
+		to the previous RFN_MODULUS cycle, wrapping... */
+		if (fn_rounded < GSM_RFN_MODULUS) {
+			/* Corner case detected: wrapping crosses GSM_MAX_FN border */
+			fn_rounded = GSM_TDMA_FN_SUB(GSM_MAX_FN, (GSM_TDMA_FN_SUB(GSM_RFN_MODULUS, fn_rounded)));
+		} else {
+			fn_rounded = GSM_TDMA_FN_SUB(fn_rounded, GSM_RFN_MODULUS);
+		}
+	}
+
+	/* The real frame number is the sum of the rounded frame number and the
+	 * relative framenumber computed via RACH */
+	return GSM_TDMA_FN_SUM(fn_rounded, rfn32);
+}
+
 /*! append range1024 encoded data to bit vector
  *  \param[out] bv Caller-provided output bit-vector
  *  \param[in] r Input Range1024 sructure */
@@ -971,6 +1019,27 @@ int gprs_tlli_type(uint32_t tlli)
 	return TLLI_RESERVED;
 }
 
+/*! Determine P-TMSI from foreign and local TLLIs
+ *
+ *  \param[in] tlli P-TMSI
+ *  \param[in] type TLLI Type we want to derive from \a p_tmsi
+ *  \returns P-TMSI or 0xffffffff on error. */
+uint32_t gprs_tlli2tmsi(uint32_t tlli)
+{
+	uint32_t ptmsi = 0xc0000000;
+
+	switch (gprs_tlli_type(tlli)) {
+	case TLLI_LOCAL:
+	case TLLI_FOREIGN:
+		break;
+	default:
+		return 0xffffffff;
+	}
+
+	ptmsi |= tlli;
+	return ptmsi;
+}
+
 /*! Determine TLLI from P-TMSI
  *  \param[in] p_tmsi P-TMSI
  *  \param[in] type TLLI Type we want to derive from \a p_tmsi
