@@ -19,7 +19,15 @@
 #include <osmocom/core/utils.h>
 #include <osmocom/sim/class_tables.h>
 
-static const uint8_t iso7816_ins_tbl[] = {
+/* Quick reminder about the "cases" as per ISO7816-3 / ETSI TS 102 221 Section 7.3.1.1
+ *
+ * Case 1: Command Length 0, Response Length 0
+ * Case 2: Command Length 0, Response Length from P3
+ * Case 3: Command Length P3, Response Length 0
+ * Case 4: Command Length P3, followed by 61xx for response length + GET RESPONSE
+ */
+
+static const uint8_t iso7816_ins_tbl[256] = {
 	[0xB0]	= 2,	/* READ BIN */
 	[0xD0]	= 3,	/* WRITE BIN */
 	[0xD6]	= 3,	/* UPDATE BIN */
@@ -113,7 +121,7 @@ static const uint8_t uicc_ins_tbl_046[256] = {
 	[0xD6]	= 3,	/* UPDATE BINARY */
 	[0xB2]	= 2,	/* READ RECORD */
 	[0xDC]	= 3,	/* UPDATE RECORD */
-	[0xA2]	= 4,	/* SEEK */
+	[0xA2]	= 4,	/* SEARCH RECORD */
 	[0x20]	= 3,	/* VERIFY PIN */
 	[0x24]	= 3,	/* CHANGE PIN */
 	[0x26]	= 3,	/* DISABLE PIN */
@@ -178,6 +186,7 @@ static int gp_cla_ins_helper(const struct osim_cla_ins_case *cic,
 {
 	uint8_t ins = hdr[1];
 	uint8_t p1 = hdr[2];
+	uint8_t p3 = hdr[4];
 
 	switch (ins) {
 	case 0xE2:	/* STORE DATA */
@@ -187,6 +196,26 @@ static int gp_cla_ins_helper(const struct osim_cla_ins_case *cic,
 		default:
 			return 3;
 		}
+		break;
+	case 0xF2:
+		/* in their infinite wisdom, GlobalPlatform re-defined the CLA 8x / INS F2 command, so one can
+		 * take a guess if it's GlobalPlatform or ETSI.  Lucikly, the P1 coding of ETSI [so far]
+		 * states all the four upper bits must be 0, while GP always has one of those bits set */
+		if (p1 & 0xF0)
+			return 4; /* GlobalPlatform v2.2 11.4.2 */
+		else
+			return 2; /* ETSI TS 102 221 V16.2.0 11.1.2 */
+		break;
+	case 0xCA:
+	case 0xCB:
+		/* in their infinite wisdom, GlobalPlatform made GET DATA a command that can be either APDU
+		 * case 2 or case 4.  As the specify Le must be 0x00, we can conclude that P3 == 0x00 must be
+		 * Le, while P3 != 0x00 must be Lc and hence case 4 */
+		if (p3 == 0x00)
+			return 2;
+		else
+			return 4;
+		break;
 	}
 	return 0;
 }
@@ -198,6 +227,7 @@ static const uint8_t uicc_ins_tbl_8ce[256] = {
 	[0xCB]		= 4,	/* RETRIEVE DATA */
 	[0xDB]		= 3,	/* SET DATA */
 	[0xAA]		= 3,	/* TERMINAL CAPABILITY */
+	[0x78]		= 4,	/* GET IDENTITY */
 };
 
 /* ETSI TS 102 221, Table 10.5, CLA = 0x80 */
@@ -206,19 +236,22 @@ static const uint8_t uicc_ins_tbl_80[256] = {
 	[0xC2]		= 4,	/* ENVELOPE */
 	[0x12]		= 2,	/* FETCH */
 	[0x14]		= 3,	/* TERMINAL RESPONSE */
+	[0x76]		= 4,	/* SUSPEND UICC */
+	[0x7A]		= 4,	/* EXCHANGE CAPABILITIES */
 };
 
 /* Card Specification v2.3.1*/
 static const uint8_t gp_ins_tbl_8ce[256] = {
 	[0xE4]		= 4,	/* DELETE */
 	[0xE2]		= 0x80,	/* STORE DATA */
-	[0xCA]		= 4,	/* GET DATA */
-	[0xCB]		= 4,	/* GET DATA */
-	[0xF2]		= 4,	/* GET STATUS */
+	[0xCA]		= 0x80,	/* GET DATA */
+	[0xCB]		= 0x80,	/* GET DATA */
+	[0xF2]		= 0x80,	/* GET STATUS */
 	[0xE6]		= 4,	/* INSTALL */
 	[0xE8]		= 4,	/* LOAD */
 	[0xD8]		= 4,	/* PUT KEY */
 	[0xF0]		= 3,	/* SET STATUS */
+	[0xC0]		= 2,	/* GET RESPONSE */
 };
 
 static const struct osim_cla_ins_case uicc_ins_case[] = {
@@ -241,6 +274,12 @@ static const struct osim_cla_ins_case uicc_ins_case[] = {
 		.cla_mask	= 0xF0,
 		.helper		= uicc046_cla_ins_helper,
 		.ins_tbl	= uicc_ins_tbl_046,
+	}, {
+		/* must be before uicc_ins_tbl_8ce below with same CLA+mask */
+		.cla		= 0x80,
+		.cla_mask	= 0xF0,
+		.helper		= gp_cla_ins_helper,
+		.ins_tbl	= gp_ins_tbl_8ce,
 	}, {
 		.cla		= 0x80,
 		.cla_mask	= 0xF0,
@@ -253,11 +292,6 @@ static const struct osim_cla_ins_case uicc_ins_case[] = {
 		.cla		= 0xE0,
 		.cla_mask	= 0xF0,
 		.ins_tbl	= uicc_ins_tbl_8ce,
-	}, {
-		.cla		= 0x80,
-		.cla_mask	= 0xF0,
-		.helper		= gp_cla_ins_helper,
-		.ins_tbl	= gp_ins_tbl_8ce,
 	}, {
 		.cla		= 0xC0,
 		.cla_mask	= 0xF0,
@@ -303,6 +337,12 @@ static const struct osim_cla_ins_case uicc_sim_ins_case[] = {
 		.cla_mask	= 0xF0,
 		.helper		= uicc046_cla_ins_helper,
 		.ins_tbl	= uicc_ins_tbl_046,
+	}, {
+		/* must be before uicc_ins_tbl_8ce below with same CLA+mask */
+		.cla		= 0x80,
+		.cla_mask	= 0xF0,
+		.helper		= gp_cla_ins_helper,
+		.ins_tbl	= gp_ins_tbl_8ce,
 	}, {
 		.cla		= 0x80,
 		.cla_mask	= 0xF0,
@@ -346,6 +386,13 @@ const uint8_t usim_ins_case[256] = {
 	[0x88]		= 4,	/* AUTHENTICATE */
 };
 
+/* https://learn.microsoft.com/en-us/windows-hardware/drivers/smartcard/discovery-process */
+static const uint8_t microsoft_discovery_ins_tbl[256] = {
+	[0xA4]	= 4,	/* SELECT FILE */
+	[0xCA]	= 2,	/* GET DATA */
+	[0xC0]	= 2,	/* GET RESPONSE */
+};
+
 int osim_determine_apdu_case(const struct osim_cla_ins_card_profile *prof,
 			     const uint8_t *hdr)
 {
@@ -370,5 +417,16 @@ int osim_determine_apdu_case(const struct osim_cla_ins_card_profile *prof,
 			return rc;
 		}
 	}
+	/* special handling for Microsoft who insists to use INS=0xCA in CLA=0x00 which is not
+	 * really part of GSM SIM, ETSI UICC or 3GPP USIM specifications, but only ISO7816. Rather than adding
+	 * it to each and every card profile, let's add the instructions listed at
+	 * https://learn.microsoft.com/en-us/windows-hardware/drivers/smartcard/discovery-process explicitly
+	 * here. They will only be used in case no more specific match was found in the actual profile above. */
+	if (cla == 0x00) {
+		rc = microsoft_discovery_ins_tbl[ins];
+		if (rc)
+			return rc;
+	}
+
 	return 0;
 }
