@@ -20,7 +20,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "../../config.h"
+#include "config.h"
 
 #include <osmocom/core/talloc.h>
 #include <osmocom/core/logging.h>
@@ -382,6 +382,10 @@ DEFUN(logging_level,
 	tgt->categories[category].enabled = 1;
 	tgt->categories[category].loglevel = level;
 
+#if !defined(EMBEDDED)
+	log_cache_update(category, 1, level);
+#endif
+
 	RET_WITH_UNLOCK(CMD_SUCCESS);
 }
 
@@ -406,6 +410,9 @@ DEFUN(logging_level_set_all, logging_level_set_all_cmd,
 
 		cat->enabled = 1;
 		cat->loglevel = level;
+#if !defined(EMBEDDED)
+		log_cache_update(i, 1, level);
+#endif
 	}
 	RET_WITH_UNLOCK(CMD_SUCCESS);
 }
@@ -1196,7 +1203,7 @@ static void gen_vty_logp_cmd_strs(struct cmd_element *cmd)
 
 /*! Register logging related commands to the VTY. Call this once from
  *  your application if you want to support those commands. */
-void logging_vty_add_cmds()
+void logging_vty_add_cmds(void)
 {
 	install_lib_element_ve(&enable_logging_cmd);
 	install_lib_element_ve(&disable_logging_cmd);
