@@ -26,7 +26,6 @@ static struct osmo_iuup_rnl_config def_configure_req = {
 		{.used = 1, .id = 2, .IPTI = 1, .subflow_sizes = {0, 0, 0} },
 	},
 	/* .delivery_err_sdu = All set to 0 (YES) by default, */
-	.IPTIs_present = true,
 	.t_init = { .t_ms = IUUP_TIMER_INIT_T_DEFAULT, .n_max = IUUP_TIMER_INIT_N_DEFAULT },
 	.t_ta = { .t_ms = IUUP_TIMER_TA_T_DEFAULT, .n_max = IUUP_TIMER_TA_N_DEFAULT },
 	.t_rc = { .t_ms = IUUP_TIMER_RC_T_DEFAULT, .n_max = IUUP_TIMER_RC_N_DEFAULT },
@@ -727,14 +726,18 @@ void test_decode_passive_init_2_rfci_no_iptis(void)
 	rnp = osmo_iuup_rnl_prim_alloc(iuup_test_ctx, OSMO_IUUP_RNL_CONFIG, PRIM_OP_REQUEST, IUUP_MSGB_SIZE);
 	rnp->u.config = def_configure_req;
 	rnp->u.config.active = false;
-	OSMO_ASSERT((rc = osmo_iuup_rnl_prim_down(iui, rnp)) == 0);
+
+	rc = osmo_iuup_rnl_prim_down(iui, rnp);
+	OSMO_ASSERT(rc == 0);
 
 	/* Send Init: */
 	tnp = osmo_iuup_tnl_prim_alloc(iuup_test_ctx, OSMO_IUUP_TNL_UNITDATA, PRIM_OP_INDICATION, IUUP_MSGB_SIZE);
 	tnp->oph.msg->l2h = msgb_put(tnp->oph.msg, sizeof(iuup_init));
 	hdr14 = (struct iuup_pdutype14_hdr *)msgb_l2(tnp->oph.msg);
 	memcpy(hdr14, iuup_init, sizeof(iuup_init));
-	OSMO_ASSERT((rc = osmo_iuup_tnl_prim_up(iui, tnp)) == 0);
+
+	rc = osmo_iuup_tnl_prim_up(iui, tnp);
+	OSMO_ASSERT(rc == 0);
 
 	osmo_iuup_instance_free(iui);
 }
